var OnlineELFViewer;
(function (OnlineELFViewer) {
    /**
     * Handles the interaction of GUI and loading an ELF file.
     */
    var FileHandler = (function () {
        function FileHandler(guiHandler) {
            this.guiHandler = guiHandler;
        }
        FileHandler.prototype.openfile = function (filelist) {
            var _this = this;
            var file = filelist[0]; /* only load one (first) file */
            var fileReader = new FileReader();
            /* once the file is loaded, the onload function is executed, passing the loaded content to the GUI handler */
            fileReader.onload = function (ev) {
                var byteBlock = (ev.target).result;
                _this.guiHandler.onFileLoad(byteBlock);
            };
            fileReader.onprogress = function (ev) { return _this.guiHandler.onFileProgress(ev); };
            fileReader.onerror = function (ev) { return _this.guiHandler.onFileErrorOrAbort(ev); };
            fileReader.onabort = function (ev) { return _this.guiHandler.onFileErrorOrAbort(ev); };
            /* start loading the file */
            fileReader.readAsArrayBuffer(file);
        };
        return FileHandler;
    })();
    OnlineELFViewer.FileHandler = FileHandler;
    /**
     * Interaction controller between GUI elements and ELF file.
     */
    var GuiHandler = (function () {
        function GuiHandler() {
            this.fileInputElement = null;
            this.resultLabelElement = null;
            this.fileStatusLabelElement = null;
            this.dropZoneElement = null;
            this.showProgramHeaderTable = null;
            this.showSectionHeaderTable = null;
            this.showSymbolTables = null;
            this.showNoteTables = null;
            this.showELFHeader = null;
            this.showProgramHeaderTableCompact = null;
            this.showSectionHeaderTableCompact = null;
            this.showSymbolTablesCompact = null;
        }
        GuiHandler.prototype.setFileInputElement = function (fileInputElement) {
            this.fileInputElement = fileInputElement;
        };
        GuiHandler.prototype.setResultLabelElement = function (resultLabelElement) {
            this.resultLabelElement = resultLabelElement;
        };
        GuiHandler.prototype.setFileStatusLabelElement = function (fileStatusLabelElement) {
            this.fileStatusLabelElement = fileStatusLabelElement;
        };
        GuiHandler.prototype.setDropZoneElement = function (dropZoneElement) {
            this.dropZoneElement = dropZoneElement;
        };
        GuiHandler.prototype.setShowProgramHeaderTableElement = function (showProgramHeaderTable) {
            this.showProgramHeaderTable = showProgramHeaderTable;
        };
        GuiHandler.prototype.setShowSectionHeaderTableElement = function (showSectionHeaderTable) {
            this.showSectionHeaderTable = showSectionHeaderTable;
        };
        GuiHandler.prototype.setShowSymbolTablesElement = function (showSymbolTables) {
            this.showSymbolTables = showSymbolTables;
        };
        GuiHandler.prototype.setShowNoteTablesElement = function (showNoteTables) {
            this.showNoteTables = showNoteTables;
        };
        GuiHandler.prototype.setShowELFHeaderElement = function (showELFHeader) {
            this.showELFHeader = showELFHeader;
        };
        GuiHandler.prototype.setShowProgramHeaderTableCompactElement = function (showProgramHeaderTableCompact) {
            this.showProgramHeaderTableCompact = showProgramHeaderTableCompact;
        };
        GuiHandler.prototype.setShowSectionHeaderTableCompactElement = function (showSectionHeaderTableCompact) {
            this.showSectionHeaderTableCompact = showSectionHeaderTableCompact;
        };
        GuiHandler.prototype.setShowSymbolTablesCompactElement = function (showSymbolTablesCompact) {
            this.showSymbolTablesCompact = showSymbolTablesCompact;
        };
        GuiHandler.prototype.getResultLabelElement = function () {
            return this.resultLabelElement;
        };
        GuiHandler.prototype.getFileStatusLabelElement = function () {
            return this.fileStatusLabelElement;
        };
        GuiHandler.prototype.initGuiDefault = function () {
            if (this.fileInputElement != null) {
                this.fileInputElement.value = '';
            }
            this.showELFHeader.checked = true;
            this.showProgramHeaderTable.checked = true;
            this.showSectionHeaderTable.checked = true;
            this.showSymbolTables.checked = true;
            this.showNoteTables.checked = true;
        };
        GuiHandler.prototype.initGuiEventHandler = function (filehandler) {
            if (this.fileInputElement != null) {
                this.fileInputElement.onchange = function (ev) {
                    filehandler.openfile((ev.target).files);
                };
            }
            if (this.dropZoneElement != null) {
                this.dropZoneElement.ondrop = function (ev) {
                    ev.preventDefault();
                    filehandler.openfile(ev.dataTransfer.files);
                };
                this.dropZoneElement.ondragover = function (ev) {
                    ev.preventDefault();
                    ev.dataTransfer.dropEffect = 'copy';
                };
            }
        };
        GuiHandler.prototype.onFileProgress = function (ev) {
            if (ev.lengthComputable) {
                this.fileStatusLabelElement.innerHTML = ev.loaded.toString() + "/" + ev.total.toString();
            }
            else {
                this.fileStatusLabelElement.innerHTML = "-";
            }
        };
        GuiHandler.prototype.onFileErrorOrAbort = function (ev) {
            this.fileStatusLabelElement.innerHTML = "File loading aborted due to an error";
        };
        GuiHandler.prototype.onFileLoad = function (byteBlock) {
            var elfFile = new ELFModel.ELFFile(byteBlock);
            var result = elfFile.load();
            if (result == ELFModel.ELFFileLoadResult.OK) {
                this.fileStatusLabelElement.innerHTML = "File successfully loaded";
                this.fileStatusLabelElement.setAttribute("style", "color:green;");
                this.showELF(elfFile);
            }
            else {
                this.fileStatusLabelElement.innerHTML = "Invalid ELF file";
                this.fileStatusLabelElement.setAttribute("style", "color:red;");
            }
        };
        GuiHandler.prototype.showELF = function (elfFile) {
            var elfViewer = new OnlineELFViewer.ELFResultViewer(elfFile);
            var resultString = "";
            if (this.showELFHeader.checked) {
                resultString = elfViewer.getFileHeaderString();
            }
            if (this.showProgramHeaderTable.checked) {
                resultString += StringUtil.NL + StringUtil.NL;
                if (this.showProgramHeaderTableCompact.checked) {
                    resultString += elfViewer.getProgramHeaderTablesCompact();
                }
                else {
                    resultString += elfViewer.getProgramHeaderTables();
                }
            }
            if (this.showSectionHeaderTable.checked) {
                resultString += StringUtil.NL + StringUtil.NL;
                if (this.showSectionHeaderTableCompact.checked) {
                    resultString += elfViewer.getSectionHeaderTablesCompact();
                }
                else {
                    resultString += elfViewer.getSectionHeaderTables();
                }
            }
            if (this.showSymbolTables.checked) {
                resultString += StringUtil.NL + StringUtil.NL;
                if (this.showSymbolTablesCompact.checked) {
                    resultString += elfViewer.getSymbolTablesCompact();
                }
                else {
                    resultString += elfViewer.getSymbolTables();
                }
            }
            if (this.showNoteTables.checked) {
                resultString += StringUtil.NL + StringUtil.NL;
                resultString += elfViewer.getNoteTables();
            }
            this.resultLabelElement.innerHTML = resultString;
        };
        return GuiHandler;
    })();
    OnlineELFViewer.GuiHandler = GuiHandler;
    var ELFResultViewer = (function () {
        function ELFResultViewer(elfFile) {
            this.elfFile = elfFile;
        }
        ELFResultViewer.prototype.getFileHeaderString = function () {
            var s = "";
            var eh = this.elfFile.elfHeader;
            s += "ELF header" + StringUtil.NL + "==========" + StringUtil.NL + StringUtil.NL;
            ;
            s += StringUtil.GetStrPadRight("Name") + StringUtil.GetStrPadRight("Offset") + StringUtil.GetStrPadRightEx("NumValue", StringUtil.DEF_PADSIZE2) + StringUtil.GetStrPadRight("Value") + StringUtil.NL;
            s += StringUtil.GetLineWithAsciiString("EI_MAG:", eh.E_ident_mag);
            s += StringUtil.GetLineWithDesc("EI_CLASS", eh.E_ident_class);
            s += StringUtil.GetLineWithDesc("EI_DATA", eh.E_ident_data);
            s += StringUtil.GetLineWithDesc("EI_VERSION", eh.E_ident_version);
            s += StringUtil.GetLineWithDesc("EI_OSABI", eh.E_ident_OsAbi);
            s += StringUtil.GetLineWithDesc("EI_OSABIVER", eh.E_ident_OsAbiVer);
            s += StringUtil.GetLineWithDesc("E_TYPE", eh.E_type);
            s += StringUtil.GetLineWithDesc("E_MACHINE", eh.E_machine);
            s += StringUtil.GetLineWithDesc("E_VERSION", eh.E_version);
            s += StringUtil.GetLineWithDesc("E_ENTRY", eh.E_Entry);
            s += StringUtil.GetLineWithDesc("E_PHOFF", eh.E_PhOff);
            s += StringUtil.GetLineWithDesc("E_SHOFF", eh.E_ShOff);
            s += StringUtil.GetLineWithDesc("E_FLAGS", eh.E_Flags);
            s += StringUtil.GetLineWithDesc("E_EHSIZE", eh.E_Ehsize);
            s += StringUtil.GetLineWithDesc("E_PHENTSIZE", eh.E_Phentsize);
            s += StringUtil.GetLineWithDesc("E_PHNUM", eh.E_Phnum);
            s += StringUtil.GetLineWithDesc("E_SHENTSIZE", eh.E_Shentsize);
            s += StringUtil.GetLineWithDesc("E_SHNUM", eh.E_Shnum);
            s += StringUtil.GetLineWithDesc("E_SHSTRNDX", eh.E_Shstrndx);
            return s;
        };
        ELFResultViewer.prototype.getProgramHeaderTables = function () {
            var s = "";
            s += "Program header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;
            for (var i = 0; i < this.elfFile.getNumOfProgramHeaderTables(); i++) {
                var pht = this.elfFile.elfProgramHeaderTables[i];
                s += "Program header table: " + i.toString() + StringUtil.NL + "-----------------------" + StringUtil.NL;
                s += StringUtil.GetLineWithDesc("P_TYPE", pht.P_Type);
                s += StringUtil.GetLineWithDesc("P_FLAGS", pht.P_Flags);
                s += StringUtil.GetLineWithDesc("P_OFFSET", pht.P_Offset);
                s += StringUtil.GetLineWithDesc("P_VADDR", pht.P_VAddr);
                s += StringUtil.GetLineWithDesc("P_PADDR", pht.P_PAddr);
                s += StringUtil.GetLineWithDesc("P_FILESZ", pht.P_FileSz);
                s += StringUtil.GetLineWithDesc("P_MEMSZ", pht.P_MemSz);
                s += StringUtil.GetLineWithDesc("P_ALIGN", pht.P_Align);
                s += StringUtil.NL;
            }
            return s;
        };
        ELFResultViewer.prototype.getProgramHeaderTablesCompact = function () {
            var s = "";
            this.elfFile.elfCompactMode = true;
            s += "Program header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;
            s += StringUtil.GetStrPadRightEx("Type", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Offset", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("VAddr", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("PAddr", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("FileSz", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("MemSz", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Align", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Flags", StringUtil.DEF_PADSIZE2) + StringUtil.NL;
            for (var i = 0; i < this.elfFile.getNumOfProgramHeaderTables(); i++) {
                var pht = this.elfFile.elfProgramHeaderTables[i];
                s += StringUtil.GetStrPadRightEx(pht.P_Type.description(), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_Offset.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_VAddr.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_PAddr.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_FileSz.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_MemSz.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_Align.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_Flags.description(), StringUtil.DEF_PADSIZE2) + StringUtil.NL;
            }
            this.elfFile.elfCompactMode = false;
            return s;
        };
        ELFResultViewer.prototype.getSectionHeaderTables = function () {
            var s = "";
            s += "Section header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;
            for (var i = 0; i < this.elfFile.getNumOfSectionHeaderTables(); i++) {
                var sht = this.elfFile.elfSectionHeaderTables[i];
                s += "Section header table: " + i.toString() + StringUtil.NL + "-----------------------" + StringUtil.NL;
                s += StringUtil.GetLineWithDesc("SH_NAME", sht.Sh_Name);
                s += StringUtil.GetLineWithDesc("SH_TYPE", sht.Sh_Type);
                s += StringUtil.GetLineWithDesc("SH_FLAGS", sht.Sh_Flags);
                s += StringUtil.GetLineWithDesc("SH_ADDR", sht.Sh_Addr);
                s += StringUtil.GetLineWithDesc("SH_OFFSET", sht.Sh_Offset);
                s += StringUtil.GetLineWithDesc("SH_SIZE", sht.Sh_Size);
                s += StringUtil.GetLineWithDesc("SH_LINK", sht.Sh_Link);
                s += StringUtil.GetLineWithDesc("SH_INFO", sht.Sh_Info);
                s += StringUtil.GetLineWithDesc("SH_ADDRALIGN", sht.Sh_Addralign);
                s += StringUtil.GetLineWithDesc("SH_ENTSIZE", sht.Sh_Entsize);
                s += StringUtil.NL;
            }
            return s;
        };
        ELFResultViewer.prototype.getSectionHeaderTablesCompact = function () {
            var s = "";
            this.elfFile.elfCompactMode = true;
            s += "Section header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;
            s += StringUtil.GetStrPadRight("Nr") +
                StringUtil.GetStrPadRightEx("Name", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRight("Type") +
                StringUtil.GetStrPadRightEx("Address", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Offset", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Size", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRight("Link") +
                StringUtil.GetStrPadRight("Info") +
                StringUtil.GetStrPadRightEx("AddrAlign", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("EntSize", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRight("Flags") + StringUtil.NL;
            for (var i = 0; i < this.elfFile.getNumOfSectionHeaderTables(); i++) {
                var sht = this.elfFile.elfSectionHeaderTables[i];
                s += StringUtil.GetStrPadRight(i.toString()) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Name.description(), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight(sht.Sh_Type.description()) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Addr.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Offset.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Size.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight(sht.Sh_Link.GetHexValueStr(8)) +
                    StringUtil.GetStrPadRight(sht.Sh_Info.GetHexValueStr(8)) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Addralign.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Entsize.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight(sht.Sh_Flags.description());
                s += StringUtil.NL;
            }
            this.elfFile.elfCompactMode = false;
            return s;
        };
        ELFResultViewer.prototype.getSymbolTables = function () {
            var s = "";
            s += "Symbol tables" + StringUtil.NL + "==============" + StringUtil.NL + StringUtil.NL;
            for (var symTabIdx = 0; symTabIdx < this.elfFile.getNumOfSymbolTables(); symTabIdx++) {
                var symTab = this.elfFile.elfSymbolTables[symTabIdx];
                var refSecTable = symTab.getRefSectionTable();
                s += "Symbol table of section table " + refSecTable.getName() + " (Index " + refSecTable.headerIndex.toString() + ") with " +
                    symTab.getNumOfEntries().toString() + " entries" + StringUtil.NL + "-----------------------" + StringUtil.NL;
                for (var symEntryIdx = 0; symEntryIdx < symTab.getNumOfEntries(); symEntryIdx++) {
                    var symEntry = symTab.symTabEntries[symEntryIdx];
                    s += "Symbol entry: " + symEntryIdx.toString() + StringUtil.NL;
                    s += StringUtil.GetLineWithDesc("SH_NAME", symEntry.St_name);
                    s += StringUtil.GetLineWithDesc("SH_VALUE", symEntry.St_value);
                    s += StringUtil.GetLineWithDesc("SH_SIZE", symEntry.St_size);
                    s += StringUtil.GetLineWithDesc("SH_INFO", symEntry.St_info);
                    s += StringUtil.GetLineWithDesc("SH_OTHER", symEntry.St_other);
                    s += StringUtil.GetLineWithDesc("SH_SHNDX", symEntry.St_shndx);
                    s += StringUtil.NL;
                }
            }
            return s;
        };
        ELFResultViewer.prototype.getSymbolTablesCompact = function () {
            var s = "";
            this.elfFile.elfCompactMode = true;
            s += "Symbol tables" + StringUtil.NL + "==============" + StringUtil.NL + StringUtil.NL;
            for (var symTabIdx = 0; symTabIdx < this.elfFile.getNumOfSymbolTables(); symTabIdx++) {
                var symTab = this.elfFile.elfSymbolTables[symTabIdx];
                var refSecTable = symTab.getRefSectionTable();
                s += "Symbol table of section table " + refSecTable.getName() + " (Index " + refSecTable.headerIndex.toString() + ") with " +
                    symTab.getNumOfEntries().toString() + " entries" + StringUtil.NL + "-----------------------" + StringUtil.NL;
                s += StringUtil.GetStrPadRight("Nr") +
                    StringUtil.GetStrPadRightEx("Value", StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx("Size", StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx("Info (Binding|Type)", StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight("Other") +
                    StringUtil.GetStrPadRight("Shndx") +
                    StringUtil.GetStrPadRight("Name") + StringUtil.NL;
                for (var symEntryIdx = 0; symEntryIdx < symTab.getNumOfEntries(); symEntryIdx++) {
                    var symEntry = symTab.symTabEntries[symEntryIdx];
                    s += StringUtil.GetStrPadRight(symEntryIdx.toString()) +
                        StringUtil.GetStrPadRightEx(symEntry.St_value.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                        StringUtil.GetStrPadRightEx(symEntry.St_size.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                        StringUtil.GetStrPadRightEx(symEntry.St_info.description(), StringUtil.DEF_PADSIZE2) +
                        StringUtil.GetStrPadRight(symEntry.St_other.description()) +
                        StringUtil.GetStrPadRight(symEntry.St_shndx.GetHexValueStr(4)) +
                        StringUtil.GetStrPadRight(symEntry.St_name.description());
                    s += StringUtil.NL;
                }
            }
            this.elfFile.elfCompactMode = false;
            return s;
        };
        ELFResultViewer.prototype.getNoteTables = function () {
            var s = "";
            s += "Note tables" + StringUtil.NL + "==============" + StringUtil.NL + StringUtil.NL;
            for (var noteTabIdx = 0; noteTabIdx < this.elfFile.getNumOfNoteTables(); noteTabIdx++) {
                var noteTab = this.elfFile.elfNoteTables[noteTabIdx];
                var refSecTable = noteTab.getRefSectionTable();
                s += "Note table of section table " + refSecTable.getName() + " (Index " + refSecTable.headerIndex.toString() + ") with " +
                    noteTab.getNumOfEntries().toString() + " entries" + StringUtil.NL + "-----------------------" + StringUtil.NL;
                for (var noteEntryIdx = 0; noteEntryIdx < noteTab.getNumOfEntries(); noteEntryIdx++) {
                    var noteEntry = noteTab.noteTabEntries[noteEntryIdx];
                    //s += "Symbol entry: " + noteEntry.getName();
                    s += StringUtil.GetLineWithDesc("Name", noteEntry.NameElement);
                    s += StringUtil.GetLineWithDesc("Type", noteEntry.Type);
                    s += StringUtil.GetLineWithByteArray("Descriptor", noteEntry.DescElement, noteEntry.getDesc());
                }
            }
            if (this.elfFile.getNumOfNoteTables() == 0) {
                s += "- Note table does not exist -";
            }
            return s;
        };
        return ELFResultViewer;
    })();
    OnlineELFViewer.ELFResultViewer = ELFResultViewer;
    var StringUtil = (function () {
        function StringUtil() {
        }
        StringUtil.GetHexStr = function (n, paddingLength) {
            return "0x" + ("00000000" + n.toString(16).toUpperCase()).slice(-paddingLength);
        };
        StringUtil.GetAsciiString = function (n) {
            var s = "";
            return String.fromCharCode((n & 0xFF000000) >> 24) + String.fromCharCode((n & 0x00FF0000) >> 16) +
                String.fromCharCode((n & 0x0000FF00) >> 8) + String.fromCharCode((n & 0x000000FF));
        };
        StringUtil.GetByteArrayString = function (values) {
            var s = "";
            values.forEach(function (element) {
                s += "0x" + ("00" + element.toString(16).toUpperCase()).slice(-2);
                s += " ";
            });
            return s;
        };
        StringUtil.GetStrPadRight = function (s) {
            return StringUtil.GetStrPadRightEx(s, StringUtil.DEF_PADSIZE);
        };
        StringUtil.GetStrPadRightEx = function (s, l) {
            var padding = l - s.length;
            while (padding > 0) {
                s = s + this.SP;
                padding--;
            }
            return s;
        };
        StringUtil.GetLineWithDesc = function (elemName, elemELF) {
            var s = StringUtil.GetStrPadRight(elemName) +
                StringUtil.GetStrPadRightEx(StringUtil.GetHexStr(elemELF.offset, 8), StringUtil.DEF_PADSIZE) +
                StringUtil.GetStrPadRightEx(elemELF.GetHexValueStr(elemELF.valueSizeInBytes * 2), StringUtil.DEF_PADSIZE2) +
                elemELF.description() + this.NL;
            return s;
        };
        StringUtil.GetLineWithAsciiString = function (elemName, elemELF) {
            var s = StringUtil.GetStrPadRight(elemName) +
                StringUtil.GetStrPadRightEx(StringUtil.GetHexStr(elemELF.offset, 8), StringUtil.DEF_PADSIZE) +
                StringUtil.GetStrPadRightEx(elemELF.GetHexValueStr(elemELF.valueSizeInBytes * 2), StringUtil.DEF_PADSIZE2) +
                StringUtil.GetAsciiString(elemELF.value) + this.NL;
            return s;
        };
        StringUtil.GetLineWithByteArray = function (elemName, elemELF, byteArray) {
            var s = StringUtil.GetStrPadRight(elemName) +
                StringUtil.GetStrPadRightEx(StringUtil.GetHexStr(elemELF.offset, 8), StringUtil.DEF_PADSIZE) +
                StringUtil.GetStrPadRightEx(elemELF.GetHexValueStr(elemELF.valueSizeInBytes * 2), StringUtil.DEF_PADSIZE2) +
                StringUtil.GetByteArrayString(byteArray) + this.NL;
            return s;
        };
        StringUtil.NL = "<br />";
        StringUtil.SP = "&nbsp;";
        StringUtil.DEF_PADSIZE = 15;
        StringUtil.DEF_PADSIZE2 = 20;
        return StringUtil;
    })();
    OnlineELFViewer.StringUtil = StringUtil;
})(OnlineELFViewer || (OnlineELFViewer = {}));
window.onload = function () {
    var guiHandler = new OnlineELFViewer.GuiHandler();
    guiHandler.setFileInputElement((document.getElementById("browsefile")));
    guiHandler.setResultLabelElement((document.getElementById("resultlabel")));
    guiHandler.setFileStatusLabelElement((document.getElementById("fileStatusLabel")));
    guiHandler.setDropZoneElement((document.getElementById("dropzone")));
    guiHandler.setShowProgramHeaderTableElement((document.getElementById("showProgramHeaderTable")));
    guiHandler.setShowSectionHeaderTableElement((document.getElementById("showSectionHeaderTable")));
    guiHandler.setShowSymbolTablesElement((document.getElementById("showSymbolTables")));
    guiHandler.setShowNoteTablesElement((document.getElementById("showNoteTables")));
    guiHandler.setShowELFHeaderElement((document.getElementById("showELFHeader")));
    guiHandler.setShowProgramHeaderTableCompactElement((document.getElementById("ElfProgramHeaderTableCompact")));
    guiHandler.setShowSectionHeaderTableCompactElement((document.getElementById("ElfSectionHeaderTableCompact")));
    guiHandler.setShowSymbolTablesCompactElement((document.getElementById("ElfSymbolTableCompact")));
    guiHandler.initGuiDefault();
    var fh = new OnlineELFViewer.FileHandler(guiHandler);
    guiHandler.initGuiEventHandler(fh);
};
//# sourceMappingURL=onlineelfviewer.js.map