﻿
namespace OnlineELFViewer
{
    /**
     * Handles the interaction of GUI and loading an ELF file.
     */
    export class FileHandler
    {
        private guiHandler: IFileGuiHandler;

        constructor(guiHandler: IFileGuiHandler)
        {
            this.guiHandler = guiHandler;
        }

        openfile(filelist: FileList)
        {
            let file = filelist[0]; /* only load one (first) file */
            let fileReader = new FileReader();

            /* once the file is loaded, the onload function is executed, passing the loaded content to the GUI handler */
            fileReader.onload = (ev: Event) =>
            {
                let byteBlock: ArrayBuffer = (<FileReader>(ev.target)).result;
                this.guiHandler.onFileLoad(byteBlock);
            }

            fileReader.onprogress = (ev: ProgressEvent) => this.guiHandler.onFileProgress(ev);
            fileReader.onerror = (ev : Event) => this.guiHandler.onFileErrorOrAbort(ev);
            fileReader.onabort = (ev : Event) => this.guiHandler.onFileErrorOrAbort(ev);

            /* start loading the file */
            fileReader.readAsArrayBuffer(file);
        }
    }

    /**
     * Interface to handle events triggered by the file loading process.
     */
    interface IFileGuiHandler
    {
        onFileProgress(ev: ProgressEvent);
        onFileErrorOrAbort(ev: Event);
        onFileLoad(byteBlock: ArrayBuffer);
    }

    /**
     * Interaction controller between GUI elements and ELF file.
     */
    export class GuiHandler implements IFileGuiHandler
    {
        private fileInputElement: HTMLInputElement = null;
        private resultLabelElement: HTMLLabelElement = null;
        private fileStatusLabelElement: HTMLLabelElement = null;
        private dropZoneElement: HTMLDivElement = null;
        private showProgramHeaderTable: HTMLInputElement = null;
        private showSectionHeaderTable: HTMLInputElement = null;
        private showSymbolTables: HTMLInputElement = null;
        private showNoteTables: HTMLInputElement = null;
        private showELFHeader: HTMLInputElement = null;

        private showProgramHeaderTableCompact: HTMLInputElement = null;
        private showSectionHeaderTableCompact: HTMLInputElement = null;
        private showSymbolTablesCompact: HTMLInputElement = null;

        public setFileInputElement(fileInputElement: HTMLInputElement)
        {
            this.fileInputElement = fileInputElement;
        }

        public setResultLabelElement(resultLabelElement: HTMLLabelElement)
        {
            this.resultLabelElement = resultLabelElement;
        }

        public setFileStatusLabelElement(fileStatusLabelElement: HTMLLabelElement)
        {
            this.fileStatusLabelElement = fileStatusLabelElement;
        }

        public setDropZoneElement(dropZoneElement: HTMLDivElement)
        {
            this.dropZoneElement = dropZoneElement;
        }

        public setShowProgramHeaderTableElement(showProgramHeaderTable: HTMLInputElement)
        {
            this.showProgramHeaderTable = showProgramHeaderTable;
        }

        public setShowSectionHeaderTableElement(showSectionHeaderTable: HTMLInputElement)
        {
            this.showSectionHeaderTable = showSectionHeaderTable;
        }

        public setShowSymbolTablesElement(showSymbolTables: HTMLInputElement)
        {
            this.showSymbolTables = showSymbolTables;
        }

        public setShowNoteTablesElement(showNoteTables: HTMLInputElement)
        {
            this.showNoteTables = showNoteTables;
        }

        public setShowELFHeaderElement(showELFHeader: HTMLInputElement)
        {
            this.showELFHeader = showELFHeader;
        }

        public setShowProgramHeaderTableCompactElement(showProgramHeaderTableCompact: HTMLInputElement)
        {
            this.showProgramHeaderTableCompact = showProgramHeaderTableCompact;
        }

        public setShowSectionHeaderTableCompactElement(showSectionHeaderTableCompact: HTMLInputElement)
        {
            this.showSectionHeaderTableCompact = showSectionHeaderTableCompact;
        }

        public setShowSymbolTablesCompactElement(showSymbolTablesCompact: HTMLInputElement)
        {
            this.showSymbolTablesCompact = showSymbolTablesCompact;
        }

        public getResultLabelElement() : HTMLLabelElement
        {
            return this.resultLabelElement;
        }

        public getFileStatusLabelElement(): HTMLLabelElement
        {
            return this.fileStatusLabelElement;
        }

        public initGuiDefault()
        {
            if (this.fileInputElement != null)
            {
                this.fileInputElement.value = '';
            }
            this.showELFHeader.checked = true;
            this.showProgramHeaderTable.checked = true;
            this.showSectionHeaderTable.checked = true;
            this.showSymbolTables.checked = true;
            this.showNoteTables.checked = true;
        }

        public initGuiEventHandler(filehandler: FileHandler)
        {
            if (this.fileInputElement != null)
            {
                this.fileInputElement.onchange = (ev: Event) =>
                {
                    filehandler.openfile((<HTMLInputElement>(ev.target)).files);
                }
            }

            if (this.dropZoneElement != null)
            {
                this.dropZoneElement.ondrop = (ev: DragEvent) =>
                {
                    ev.preventDefault();
                    filehandler.openfile(ev.dataTransfer.files);
                }

                this.dropZoneElement.ondragover = (ev: DragEvent) =>
                {
                    ev.preventDefault();
                    ev.dataTransfer.dropEffect = 'copy';
                }
            }
        }

        public onFileProgress(ev: ProgressEvent)
        {
            if (ev.lengthComputable)
            {
                this.fileStatusLabelElement.innerHTML = ev.loaded.toString() + "/" + ev.total.toString();
            }
            else
            {
                this.fileStatusLabelElement.innerHTML = "-";
            }
        }

        public onFileErrorOrAbort(ev: Event)
        {
            this.fileStatusLabelElement.innerHTML = "File loading aborted due to an error";
        }

        public onFileLoad(byteBlock: ArrayBuffer)
        {
            let elfFile: ELFModel.ELFFile = new ELFModel.ELFFile(byteBlock);
            let result: ELFModel.ELFFileLoadResult = elfFile.load();

            if (result == ELFModel.ELFFileLoadResult.OK)
            {
                this.fileStatusLabelElement.innerHTML = "File successfully loaded";
                this.fileStatusLabelElement.setAttribute("style", "color:green;");
                this.showELF(elfFile);
            }
            else
            {
                this.fileStatusLabelElement.innerHTML = "Invalid ELF file";
                this.fileStatusLabelElement.setAttribute("style", "color:red;");
            }
        }

        public showELF(elfFile: ELFModel.ELFFile)
        {
            let elfViewer: OnlineELFViewer.ELFResultViewer = new OnlineELFViewer.ELFResultViewer(elfFile);
            let resultString: string = "";

            if (this.showELFHeader.checked)
            {
                resultString = elfViewer.getFileHeaderString();
            }

            if (this.showProgramHeaderTable.checked)
            {
                resultString += StringUtil.NL + StringUtil.NL;
                if (this.showProgramHeaderTableCompact.checked)
                {
                    resultString += elfViewer.getProgramHeaderTablesCompact();
                }
                else
                {
                    resultString += elfViewer.getProgramHeaderTables();
                }
            }

            if (this.showSectionHeaderTable.checked)
            {
                resultString += StringUtil.NL + StringUtil.NL;
                if (this.showSectionHeaderTableCompact.checked)
                {
                    resultString += elfViewer.getSectionHeaderTablesCompact();
                }
                else
                {
                    resultString += elfViewer.getSectionHeaderTables();
                }
            }

            if (this.showSymbolTables.checked)
            {
                resultString += StringUtil.NL + StringUtil.NL;
                if (this.showSymbolTablesCompact.checked)
                {
                    resultString += elfViewer.getSymbolTablesCompact();
                }
                else
                {
                    resultString += elfViewer.getSymbolTables();
                }
            }

            if (this.showNoteTables.checked)
            {
                resultString += StringUtil.NL + StringUtil.NL;
                resultString += elfViewer.getNoteTables();
            }

            this.resultLabelElement.innerHTML = resultString;
        }
    }

    export class ELFResultViewer
    {
        private elfFile: ELFModel.ELFFile;

        constructor(elfFile: ELFModel.ELFFile)
        {
            this.elfFile = elfFile;
        }

        public getFileHeaderString(): string
        {
            let s: string = "";
            let eh: ELFModel.ELFHeader = this.elfFile.elfHeader;

            s += "ELF header" + StringUtil.NL + "==========" + StringUtil.NL + StringUtil.NL;;
            s += StringUtil.GetStrPadRight("Name") + StringUtil.GetStrPadRight("Offset") + StringUtil.GetStrPadRightEx("NumValue", StringUtil.DEF_PADSIZE2) + StringUtil.GetStrPadRight("Value") + StringUtil.NL;
            s += StringUtil.GetLineWithAsciiString("EI_MAG:", eh.E_ident_mag);
            s += StringUtil.GetLineWithDesc("EI_CLASS", eh.E_ident_class);
            s += StringUtil.GetLineWithDesc("EI_DATA", eh.E_ident_data);
            s += StringUtil.GetLineWithDesc("EI_VERSION", eh.E_ident_version);
            s += StringUtil.GetLineWithDesc("EI_OSABI", eh.E_ident_OsAbi);
            s += StringUtil.GetLineWithDesc("EI_OSABIVER", eh.E_ident_OsAbiVer);
            s += StringUtil.GetLineWithDesc("E_TYPE", eh.E_type);
            s += StringUtil.GetLineWithDesc("E_MACHINE", eh.E_machine);
            s += StringUtil.GetLineWithDesc("E_VERSION", eh.E_version);
            s += StringUtil.GetLineWithDesc("E_ENTRY", eh.E_Entry);
            s += StringUtil.GetLineWithDesc("E_PHOFF", eh.E_PhOff);
            s += StringUtil.GetLineWithDesc("E_SHOFF", eh.E_ShOff);
            s += StringUtil.GetLineWithDesc("E_FLAGS", eh.E_Flags);
            s += StringUtil.GetLineWithDesc("E_EHSIZE", eh.E_Ehsize);
            s += StringUtil.GetLineWithDesc("E_PHENTSIZE", eh.E_Phentsize);
            s += StringUtil.GetLineWithDesc("E_PHNUM", eh.E_Phnum);
            s += StringUtil.GetLineWithDesc("E_SHENTSIZE", eh.E_Shentsize);
            s += StringUtil.GetLineWithDesc("E_SHNUM", eh.E_Shnum);
            s += StringUtil.GetLineWithDesc("E_SHSTRNDX", eh.E_Shstrndx);

            return s;
        }

        public getProgramHeaderTables(): string
        {
            let s: string = "";

            s += "Program header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;
            for (let i: number = 0; i < this.elfFile.getNumOfProgramHeaderTables(); i++)
            {
                let pht = this.elfFile.elfProgramHeaderTables[i];

                s += "Program header table: " + i.toString() + StringUtil.NL + "-----------------------" + StringUtil.NL;

                s += StringUtil.GetLineWithDesc("P_TYPE", pht.P_Type);
                s += StringUtil.GetLineWithDesc("P_FLAGS", pht.P_Flags);
                s += StringUtil.GetLineWithDesc("P_OFFSET", pht.P_Offset);
                s += StringUtil.GetLineWithDesc("P_VADDR", pht.P_VAddr);
                s += StringUtil.GetLineWithDesc("P_PADDR", pht.P_PAddr);
                s += StringUtil.GetLineWithDesc("P_FILESZ", pht.P_FileSz);
                s += StringUtil.GetLineWithDesc("P_MEMSZ", pht.P_MemSz);
                s += StringUtil.GetLineWithDesc("P_ALIGN", pht.P_Align);
                s += StringUtil.NL;
            }

            return s;
        }

        public getProgramHeaderTablesCompact(): string
        {
            let s: string = "";
            this.elfFile.elfCompactMode = true;

            s += "Program header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;

            s += StringUtil.GetStrPadRightEx("Type", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Offset", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("VAddr", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("PAddr", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("FileSz", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("MemSz", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Align", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Flags", StringUtil.DEF_PADSIZE2) + StringUtil.NL;

            for (let i: number = 0; i < this.elfFile.getNumOfProgramHeaderTables(); i++)
            {
                let pht = this.elfFile.elfProgramHeaderTables[i];

                s += StringUtil.GetStrPadRightEx(pht.P_Type.description(), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_Offset.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_VAddr.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_PAddr.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_FileSz.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_MemSz.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_Align.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(pht.P_Flags.description(), StringUtil.DEF_PADSIZE2) + StringUtil.NL;
            }

            this.elfFile.elfCompactMode = false;
            return s;
        }

        public getSectionHeaderTables(): string
        {
            let s: string = "";

            s += "Section header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;
            for (let i: number = 0; i < this.elfFile.getNumOfSectionHeaderTables(); i++)
            {
                let sht = this.elfFile.elfSectionHeaderTables[i];

                s += "Section header table: " + i.toString() + StringUtil.NL + "-----------------------" + StringUtil.NL;

                s += StringUtil.GetLineWithDesc("SH_NAME", sht.Sh_Name);
                s += StringUtil.GetLineWithDesc("SH_TYPE", sht.Sh_Type);
                s += StringUtil.GetLineWithDesc("SH_FLAGS", sht.Sh_Flags);
                s += StringUtil.GetLineWithDesc("SH_ADDR", sht.Sh_Addr);
                s += StringUtil.GetLineWithDesc("SH_OFFSET", sht.Sh_Offset);
                s += StringUtil.GetLineWithDesc("SH_SIZE", sht.Sh_Size);
                s += StringUtil.GetLineWithDesc("SH_LINK", sht.Sh_Link);
                s += StringUtil.GetLineWithDesc("SH_INFO", sht.Sh_Info);
                s += StringUtil.GetLineWithDesc("SH_ADDRALIGN", sht.Sh_Addralign);
                s += StringUtil.GetLineWithDesc("SH_ENTSIZE", sht.Sh_Entsize);
                s += StringUtil.NL;
            }

            return s;
        }

        public getSectionHeaderTablesCompact(): string
        {
            let s: string = "";
            this.elfFile.elfCompactMode = true;

            s += "Section header tables" + StringUtil.NL + "=====================" + StringUtil.NL + StringUtil.NL;

            s += StringUtil.GetStrPadRight("Nr") +
                StringUtil.GetStrPadRightEx("Name", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRight("Type") +
                StringUtil.GetStrPadRightEx("Address", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Offset", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("Size", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRight("Link") +
                StringUtil.GetStrPadRight("Info") +
                StringUtil.GetStrPadRightEx("AddrAlign", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRightEx("EntSize", StringUtil.DEF_PADSIZE2) +
                StringUtil.GetStrPadRight("Flags") + StringUtil.NL;

            for (let i: number = 0; i < this.elfFile.getNumOfSectionHeaderTables(); i++)
            {
                let sht = this.elfFile.elfSectionHeaderTables[i];

                s += StringUtil.GetStrPadRight(i.toString()) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Name.description(), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight(sht.Sh_Type.description()) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Addr.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Offset.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Size.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight(sht.Sh_Link.GetHexValueStr(8)) +
                    StringUtil.GetStrPadRight(sht.Sh_Info.GetHexValueStr(8)) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Addralign.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx(sht.Sh_Entsize.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight(sht.Sh_Flags.description());
                s += StringUtil.NL;
            }

            this.elfFile.elfCompactMode = false;
            return s;
        }

        public getSymbolTables(): string
        {
            let s: string = "";

            s += "Symbol tables" + StringUtil.NL + "==============" + StringUtil.NL + StringUtil.NL;

            for (let symTabIdx: number = 0; symTabIdx < this.elfFile.getNumOfSymbolTables(); symTabIdx++)
            {
                let symTab: ELFModel.ELFSymbolTable = this.elfFile.elfSymbolTables[symTabIdx];
                let refSecTable: ELFModel.ELFSectionHeaderTable = symTab.getRefSectionTable();

                s += "Symbol table of section table " + refSecTable.getName() + " (Index " + refSecTable.headerIndex.toString() + ") with " +
                    symTab.getNumOfEntries().toString() + " entries" + StringUtil.NL + "-----------------------" + StringUtil.NL;

                for (let symEntryIdx: number = 0; symEntryIdx < symTab.getNumOfEntries(); symEntryIdx++)
                {
                    let symEntry: ELFModel.ELFSymbolTableEntry = symTab.symTabEntries[symEntryIdx];

                    s += "Symbol entry: " + symEntryIdx.toString() + StringUtil.NL;
                    s += StringUtil.GetLineWithDesc("SH_NAME", symEntry.St_name);
                    s += StringUtil.GetLineWithDesc("SH_VALUE", symEntry.St_value);
                    s += StringUtil.GetLineWithDesc("SH_SIZE", symEntry.St_size);
                    s += StringUtil.GetLineWithDesc("SH_INFO", symEntry.St_info);
                    s += StringUtil.GetLineWithDesc("SH_OTHER", symEntry.St_other);
                    s += StringUtil.GetLineWithDesc("SH_SHNDX", symEntry.St_shndx);
                    s += StringUtil.NL;
                }
            }

            return s;
        }

        public getSymbolTablesCompact(): string
        {
            let s: string = "";
            this.elfFile.elfCompactMode = true;

            s += "Symbol tables" + StringUtil.NL + "==============" + StringUtil.NL + StringUtil.NL;

            for (let symTabIdx: number = 0; symTabIdx < this.elfFile.getNumOfSymbolTables(); symTabIdx++)
            {
                let symTab: ELFModel.ELFSymbolTable = this.elfFile.elfSymbolTables[symTabIdx];
                let refSecTable: ELFModel.ELFSectionHeaderTable = symTab.getRefSectionTable();

                s += "Symbol table of section table " + refSecTable.getName() + " (Index " + refSecTable.headerIndex.toString() + ") with " +
                    symTab.getNumOfEntries().toString() + " entries" + StringUtil.NL + "-----------------------" + StringUtil.NL;

                s += StringUtil.GetStrPadRight("Nr") +
                    StringUtil.GetStrPadRightEx("Value", StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx("Size", StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRightEx("Info (Binding|Type)", StringUtil.DEF_PADSIZE2) +
                    StringUtil.GetStrPadRight("Other") +
                    StringUtil.GetStrPadRight("Shndx") +
                    StringUtil.GetStrPadRight("Name") + StringUtil.NL;

                for (let symEntryIdx: number = 0; symEntryIdx < symTab.getNumOfEntries(); symEntryIdx++)
                {
                    let symEntry: ELFModel.ELFSymbolTableEntry = symTab.symTabEntries[symEntryIdx];

                    s += StringUtil.GetStrPadRight(symEntryIdx.toString()) +
                        StringUtil.GetStrPadRightEx(symEntry.St_value.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                        StringUtil.GetStrPadRightEx(symEntry.St_size.GetHexValueStr(8), StringUtil.DEF_PADSIZE2) +
                        StringUtil.GetStrPadRightEx(symEntry.St_info.description(), StringUtil.DEF_PADSIZE2) +
                        StringUtil.GetStrPadRight(symEntry.St_other.description()) +
                        StringUtil.GetStrPadRight(symEntry.St_shndx.GetHexValueStr(4)) +
                        StringUtil.GetStrPadRight(symEntry.St_name.description());
                    s += StringUtil.NL;
                }

            }

            this.elfFile.elfCompactMode = false;
            return s;
        }

        public getNoteTables(): string
        {
            let s: string = "";

            s += "Note tables" + StringUtil.NL + "==============" + StringUtil.NL + StringUtil.NL;

            for (let noteTabIdx: number = 0; noteTabIdx < this.elfFile.getNumOfNoteTables(); noteTabIdx++)
            {
                let noteTab: ELFModel.ELFNoteTable = this.elfFile.elfNoteTables[noteTabIdx];
                let refSecTable: ELFModel.ELFSectionHeaderTable = noteTab.getRefSectionTable();

                s += "Note table of section table " + refSecTable.getName() + " (Index " + refSecTable.headerIndex.toString() + ") with " +
                    noteTab.getNumOfEntries().toString() + " entries" + StringUtil.NL + "-----------------------" + StringUtil.NL;

                for (let noteEntryIdx: number = 0; noteEntryIdx < noteTab.getNumOfEntries(); noteEntryIdx++)
                {
                    let noteEntry: ELFModel.ELFNoteTableEntry = noteTab.noteTabEntries[noteEntryIdx];

                    //s += "Symbol entry: " + noteEntry.getName();
                    s += StringUtil.GetLineWithDesc("Name", noteEntry.NameElement);
                    s += StringUtil.GetLineWithDesc("Type", noteEntry.Type);
                    s += StringUtil.GetLineWithByteArray("Descriptor", noteEntry.DescElement, noteEntry.getDesc());
                }

            }

            if (this.elfFile.getNumOfNoteTables() == 0)
            {
                s += "- Note table does not exist -";
            }

            return s;
        }
    }

    export class StringUtil
    {
        public static NL: string = "<br />";
        public static SP: string = "&nbsp;";
        public static DEF_PADSIZE: number = 15;
        public static DEF_PADSIZE2: number = 20;

        public static GetHexStr(n: number, paddingLength: number): string
        {
            return "0x" + ("00000000" + n.toString(16).toUpperCase()).slice(-paddingLength);
        }

        public static GetAsciiString(n: number): string
        {
            let s: string = "";

            return String.fromCharCode((n & 0xFF000000) >> 24) + String.fromCharCode((n & 0x00FF0000) >> 16) +
                   String.fromCharCode((n & 0x0000FF00) >> 8) + String.fromCharCode((n & 0x000000FF));
        }

        public static GetByteArrayString(values: number[]): string
        {
            let s: string = "";

            values.forEach((element) =>
            {
                s += "0x" + ("00" + element.toString(16).toUpperCase()).slice(-2);
                s += " ";
            });

            return s;
        }

        public static GetStrPadRight(s: string): string
        {
            return StringUtil.GetStrPadRightEx(s, StringUtil.DEF_PADSIZE);
        }

        public static GetStrPadRightEx(s: string, l: number): string
        {
            let padding = l - s.length;
            while (padding > 0)
            {
                s = s + this.SP;
                padding--;
            }
            return s;
        }

        public static GetLineWithDesc(elemName: string, elemELF: ELFModel.ELFElement) : string
        {
            let s: string =
                StringUtil.GetStrPadRight(elemName) +
                StringUtil.GetStrPadRightEx(StringUtil.GetHexStr(elemELF.offset, 8), StringUtil.DEF_PADSIZE) +
                StringUtil.GetStrPadRightEx(elemELF.GetHexValueStr(elemELF.valueSizeInBytes * 2), StringUtil.DEF_PADSIZE2) +
                elemELF.description() + this.NL;
            return s;
        }

        public static GetLineWithAsciiString(elemName: string, elemELF: ELFModel.ELFElement): string
        {
            let s: string =
                StringUtil.GetStrPadRight(elemName) +
                StringUtil.GetStrPadRightEx(StringUtil.GetHexStr(elemELF.offset, 8), StringUtil.DEF_PADSIZE) +
                StringUtil.GetStrPadRightEx(elemELF.GetHexValueStr(elemELF.valueSizeInBytes * 2), StringUtil.DEF_PADSIZE2) +
                StringUtil.GetAsciiString(elemELF.value) + this.NL;
            return s;
        }

        public static GetLineWithByteArray(elemName: string, elemELF: ELFModel.ELFElement, byteArray: number[]): string
        {
            let s: string =
                StringUtil.GetStrPadRight(elemName) +
                StringUtil.GetStrPadRightEx(StringUtil.GetHexStr(elemELF.offset, 8), StringUtil.DEF_PADSIZE) +
                StringUtil.GetStrPadRightEx(elemELF.GetHexValueStr(elemELF.valueSizeInBytes * 2), StringUtil.DEF_PADSIZE2) +
                StringUtil.GetByteArrayString(byteArray) + this.NL;
            return s;
        }
        
    }
}

window.onload = () =>
{
    let guiHandler = new OnlineELFViewer.GuiHandler();
    guiHandler.setFileInputElement(<HTMLInputElement>(document.getElementById("browsefile")));
    guiHandler.setResultLabelElement(<HTMLLabelElement>(document.getElementById("resultlabel")));
    guiHandler.setFileStatusLabelElement(<HTMLLabelElement>(document.getElementById("fileStatusLabel")));
    guiHandler.setDropZoneElement(<HTMLDivElement>(document.getElementById("dropzone")));
    guiHandler.setShowProgramHeaderTableElement(<HTMLInputElement>(document.getElementById("showProgramHeaderTable")));
    guiHandler.setShowSectionHeaderTableElement(<HTMLInputElement>(document.getElementById("showSectionHeaderTable")));
    guiHandler.setShowSymbolTablesElement(<HTMLInputElement>(document.getElementById("showSymbolTables")));
    guiHandler.setShowNoteTablesElement(<HTMLInputElement>(document.getElementById("showNoteTables")));
    guiHandler.setShowELFHeaderElement(<HTMLInputElement>(document.getElementById("showELFHeader")));

    guiHandler.setShowProgramHeaderTableCompactElement(<HTMLInputElement>(document.getElementById("ElfProgramHeaderTableCompact")));
    guiHandler.setShowSectionHeaderTableCompactElement(<HTMLInputElement>(document.getElementById("ElfSectionHeaderTableCompact")));
    guiHandler.setShowSymbolTablesCompactElement(<HTMLInputElement>(document.getElementById("ElfSymbolTableCompact")));

    guiHandler.initGuiDefault();

    let fh = new OnlineELFViewer.FileHandler(guiHandler);

    guiHandler.initGuiEventHandler(fh);
};
